@@ -2,11 +2,12 @@
 import logging
 import signal
 import sys
+import traceback
 
 import six
 import yaml
 
-__version__ = "0.2.0"
+__version__ = "0.2.1"
 protocol_version = 2
 
 # time to wait for msg_received
@@ -271,7 +272,8 @@ class DAPPServer(DAPPCommunicator):
             self.proc.stdin.write(msg)
             self.proc.stdin.flush()
         except IOError as e:
-            msg = 'Error writing data to PingPong subprocess: {0}\n'.format(e)
+            msg = 'Error writing data to PingPong subprocess: {e}\n'.\
+                format(e=format_exc())
             out = self._try_read_subprocess_error()
             if out:
                 msg += 'Subprocess output:\n' + out
@@ -387,19 +389,17 @@ class DAPPClient(DAPPCommunicator):
         try:
             msg = self.recv_msg(allowed_types=['run'])
         except DAPPException as e:
-            self.send_msg_failed(ctxt=None, fail_desc=str(e))
+            self.send_msg_failed(ctxt=None, fail_desc=traceback.format_exc())
             sys.exit(1)
         fail_desc = None
-        if fail_desc is not None:
-            self.send_msg_failed(ctxt, fail_desc)
-            sys.exit(1)
 
         # actually run
         ctxt = msg.get('ctxt', {})
         try:
             run_result = self.run(ctxt)
         except BaseException as e:
-            fail_desc = 'PingPong run method ended with an exception:\n{e}'.format(e=e)
+            fail_desc = 'PingPong run method ended with an exception:\n{e}'.\
+                format(e=traceback.format_exc())
             self.send_msg_failed(ctxt, fail_desc)
             sys.exit(1)
 
@@ -407,7 +407,7 @@ class DAPPClient(DAPPCommunicator):
             fail_desc = ['PingPong run method ended with unexpected result:',
                 str(run_result),
                 '(expected 2-tuple)']
-            self.send_msg_failed(ctxt, fail_desc)
+            self.send_msg_failed(ctxt, ' '.join(fail_desc))
             sys.exit(1)
 
         # send "finished" message to DevAssistant
